;*************************************************
; This script applies differents masks on data.
; It assumes you already ran the 
; "gREC_create_maskedData_CROCUS.ncl" script in 
; order to create the new mask.
;
; Created On: 2023
; Created By: S. Buarque
;
;*************************************************
load "goodtime.ncl"
load "goodunits.ncl"
load "fct_res_NH.ncl"
load "fct_res_Satellite.ncl"
load "fct_get_Levels.ncl"
load "shapefile_utils.ncl"
;
; *************************************************************************************
; Terrestrial Snow Cover (from Methods and data)
; L. Mudryk 1 , A. Elias Chereque 2 , C. Derksen 1 , K. Luojus 3 , and B. Decharme 4
; https://doi.org/10.25923/16xy-9h55
;
; It is matter of monthly anomalies of total areal snow cover over land for a given 
; Arctic sector (North America or Eurasia, > 60° N).
; It are computed and standardized relative to the 1981-2010 period (each observation 
; differenced from the mean and divided by the standard deviation and thus unitless).
; *************************************************************************************
;
;*************************************************
;*************************************************
undef("plt_Fig1")
procedure plt_Fig1(pltName,lat1d,lon1d,data,dataMasked,goodLevels)
begin
   wks = gsn_open_wks("png",pltName)
   chain = str_split(pltName,"_")
   expName=chain(1)
   timeName=chain(dimsizes(chain)-1)
;
;--resources for plots
;
   YMin = toint(min(lat1d))
   YMax = toint(max(lat1d))
   XMin = toint(min(lon1d))
   XMax = toint(max(lon1d))
   res    = fct_res_Satellite(YMin,YMax,XMin,XMax)
   res@cnLevelSelectionMode = "ExplicitLevels"
   res@cnLevels = goodLevels
   res@lbBoxEndCapStyle = "TriangleHighEnd"
;
;--create two plots for panelling.
   plot    = new(2,graphic)
   res@tiMainString  = " Masked  ("+ \
                       str_join(tostring(dimsizes(dataMasked))," x ") + ")"
   plot(0) = gsn_csm_contour_map(wks,dataMasked, res)
   res@tiMainString  = " Original  (" + \
                      str_join(tostring(dimsizes(data))," x ") + ")"
   plot(1) = gsn_csm_contour_map(wks,data, res)

   resP                     = True
   resP@gsnMaximize         = True                 ; make ps/eps/pdf large
   resP@gsnPanelLabelBar    = True                 ; True, add common colorbar
   resP@gsnPanelMainString  = expName +" ("+timeName+")"
   gsn_panel(wks,plot(0:1),(/2,1/),resP)           ; now draw as one plot
   draw(plot)
   return
end
;
;*************************************************
;*************************************************
undef("plt_xZon")
procedure plt_xZon(pltName,pltMainTitle,tmp)
local xZonT
begin
;https://www.ncl.ucar.edu/Applications/Scripts/climo_4.ncl
;
;xZonT = transpose(dim_avg_n_Wrap(tmp,2))          ; (lat,time)
;
wks = gsn_open_wks("png",pltName)
gsn_define_colormap(wks,"WhiteBlueGreenYellowRed")    ; change colormap
gsn_reverse_colormap(wks)
;
fmt = "%6.2f"
space = "                           "
mini=toint(min(tmp))
maxi=toint(max(tmp))
Levels = fct_get_Levels("posLab4")
;
res                     = True            ; plot mods desired
 res@gsnDraw          = False              ; don't draw yet
 res@gsnFrame         = False              ; don't advance frame yet
res@gsnRightString      = " "

res@vpXF             	= 0.12                ; default is 0.2 (aspect ratio)
res@vpYF             	= 0.8                 ; default is 0.8
res@vpHeightF        	= 0.35                 ; default is 0.6
res@vpWidthF         	= 0.68                 ; default is 0.6

res@cnFillOn            = True           ; turn on color
res@cnLinesOn           = True            ; contour lines

res@tiMainString        = pltMainTitle
res@tiYAxisString    	= "Latitude"
res@txString            = "Min= "+sprintf(fmt,mini)+space+"Max= "+sprintf(fmt,maxi)
  
;res@lbTitleString           = "Min= "+sprintf(fmt,mini)+space+"Max= "+sprintf(fmt,maxi)
;res@lbTitleFontHeightF      =  0.015             ;--decrease the font size (default: 0.025)
  
res@tmXBMode             = "Explicit"      ; label independently
res@tmXBValues           = ispan(0,11,1)   ; x-axis --> months
res@tmXBLabels           = (/"J","F","M","A","M","J",\
                             "J","A","S","O","N","D"/)
  
res@cnLevelSelectionMode = "ExplicitLevels"  ; set explicit contour levels
res@lbBoxEndCapStyle     = "TriangleHighEnd"
res@cnLevels             = Levels
  
plot = gsn_csm_lat_time(wks, tmp, res ) 

;*************************
; plotting parameters
;*************************

; res@cnLevelSelectionMode = "ManualLevels" ; manual levels
; res@cnMinLevelValF       = -4.            ; min level
; res@cnMaxLevelValF       =  4.            ; max level
; res@cnLevelSpacingF      =  0.5           ; contour spacing

gtPat = 17                                           ; stiple
ltPat = 3                                            ; hatch

 plot = gsn_csm_lat_time(wks, tmp, res ) 
 opt  = True
 opt@gsnShadeFillType = "pattern"
 opt@gsnShadeLow      = ltPat
 opt@gsnShadeHigh     = gtPat
 plot = gsn_contour_shade(plot, -0.69, 0.25, opt)
 plot = ZeroLineContour (plot)
 draw (plot)                                ; draw the contour object
 frame (wks)                                ; advance frame

plot = gsn_csm_lat_time(wks, tmp, res ) 
return
end
;
;*************************************************
;*************************************************
undef("get_wgt")
function get_wgt(lat1D,lon1D)
local lat1D, lon1D
begin
;
nlat            = dimsizes(lat1D)
;
rad    = 4.*atan(1.)/180.
;re     = 6371220.0		; earth radius (m)
re     = 6371.22                ; (km)
rr     = re*rad
;
dlon            = abs(lon1D(2)-lon1D(1))*rr
dx              = dlon*cos(lat1D*rad)
;
dy              = new (nlat, typeof(dx))
;
dy(0)           = abs(lat1D(2)-lat1D(1))*rr
dy(1:nlat-2)    = abs(lat1D(2:nlat-1)-lat1D(0:nlat-3))*rr*0.5   
dy(nlat-1)      = abs(lat1D(nlat-1)-lat1D(nlat-2))*rr
;
area   = dx*dy                      ; cell area function of latitude only
copy_VarMeta(lat1D,area)
;
cosl   = cos(lat1D*rad)             ; simple cosine weighting
clat   = latRegWgt(lat1D, "double", 0) 
;
return([/area,cosl,clat/])
end
;
;*************************************************
;*************************************************
undef("cal_stat")
procedure cal_stat(yr, data, asciiPath) 
local yr, lat1D, lon1D
begin
  att = getVarDimNames(data)
  namY  = att(0)
  namX  = att(1)
;
  lat1D   =  data&$namY$
  lon1D   =  data&$namX$
  Ndim1D = dimsizes(lat1D)*dimsizes(lon1D)
  data1D    = new((/Ndim1D/), typeof(data))
  ind_Y   = ind(.not.ismissing(lat1D))
  ind_X   = ind(.not.ismissing(lon1D))
  n=0
  do j=0, dimsizes(ind_Y) -1
     do i=0, dimsizes(ind_X) -1
        data1D(n) = (/data(j,i)/)
        n=n+1
     end do
  end do
  xstat = dim_stat4(data1D)
  stdev = dim_stddev(data1D)
;
;--- Write data to a ascii file
;
  fmtext = sprinti("%4d ",yr) + \
           sprintf("%6.2f ",min(data1D)) + \
           sprintf("%6.2f ",max(data1D)) + \
           sprintf("%8.3f ",xstat(0))    + \
           sprintf("%10.3f ",stdev)      + \
           sprintf("%10.3f ",xstat(2))   + \
           sprintf("%10.3f ",xstat(3))
  write_table(asciiPath, "a", [/fmtext/], "%s")
  return
end
;
;*************************************************
;*************************************************
undef("get_statFile")
procedure get_statFile(asciiPath,tmp)
begin
att = getVarDimNames(tmp)
timename  = att(0)
fmtext = "year min    max    ave      stdev     skewness  kurtosis"
write_table(asciiPath, "w", [/fmtext/], "%s")
taxis = toint(tmp&$timename$)
do t=0, dimsizes(taxis)-1
   cal_stat(taxis(t),tmp(t,:,:),asciiPath)
end do 
end
;
;*************************************************
;*************************************************
undef("get_ind")
function get_ind(area,yyyy_sel,data_3D_sea_inside_extract,expe,region,cr)
local expe, region, mask_country
begin
; index of climatological period
timename = data_3D_sea_inside_extract!0
ind_clm  = ind(yyyy_sel.ge.1981 .and. yyyy_sel.le.2000)
ind_2018 = ind(yyyy_sel.eq.2018)
;
timename = data_3D_sea_inside_extract!0
latname = data_3D_sea_inside_extract!1
lonname = data_3D_sea_inside_extract!2
;
; A) Total areal snow cover --> xCopy_total_area
;
; areas--> use conform( dims, r , ndim)
;
; crée un tableau conforme à la forme des dimensions données (dims)
; en propageant les valeurs de r, ndim, étant sont indice/dimension dans dims. 
; Si r est un scalaire, alors ndim peut avoir la valeur spéciale de -1 (voir ci-dessous).
;
ww = conform(data_3D_sea_inside_extract, area, 1)
copy_VarMeta(data_3D_sea_inside_extract, ww)
wwnorm = ww/sum(ww)
copy_VarMeta(ww, wwnorm)
print("-- ww            : min= "+min(ww)+" max= "+max(ww))
print("-- sum(wwnorm)   : "+sum(wwnorm))
print("  ")
;
; attribute elementary areas to xCopy and calculate its sum
xCopy=new((/dimsizes(data_3D_sea_inside_extract)/),typeof(ww), default_fillvalue("integer"))
copy_VarMeta(data_3D_sea_inside_extract, xCopy)
xCopy_total_area=new((/dimsizes(yyyy_sel)/),typeof(ww), default_fillvalue("integer"))
copy_VarMeta(data_3D_sea_inside_extract&$timename$, xCopy_total_area)
do nT=0,dimsizes(yyyy_sel)-1
   xCopy(nT,:,:) = todouble(data_3D_sea_inside_extract(nT,:,:)) * ww(nT,:,:)
   xCopy_total_area(nT) = sum(xCopy(nT,:,:))
end do
fmt = "%10.0f"
print("-- xCopy                (km2) rank= "+dimsizes(dimsizes(xCopy))\
		+"  Min= "+sprintf(fmt,min(xCopy)) \
		+"  Max= "+sprintf(fmt,max(xCopy)) )
print("-- xCopy_total_area     (km2) rank= "+dimsizes(dimsizes(xCopy_total_area))\
		+"  Min= "+sprintf(fmt,min(xCopy_total_area)) \
		+"  Max= "+sprintf(fmt,max(xCopy_total_area)) )
;
; B) Anomalies of total areal snow cover computed and standardized relative to the 
;    1981-2010 period (each observation differenced from the mean and divided by the 
;    standard deviation and thus unitless).
;
fmt = "%5.2f"
print("  ")
print("1) Anomalies on time by elementary area --> result(time,lat,lon)")
print("  ")
xCopy_avg_on_time    = dim_avg_n_Wrap(xCopy(ind_clm,:,:), 0)	  ; (lat,lon)
xCopy_avg_on_time_1d = ndtooned(xCopy_avg_on_time)                ; to one-dimension
xCopy_avg_on_time_stdev = dim_stddev(xCopy_avg_on_time_1d)
print("-- xCopy_avg_on_time    (km2) rank= "+dimsizes(dimsizes(xCopy_avg_on_time)) \
		+"  Min= "+sprintf(fmt,min(xCopy_avg_on_time)) \
		+"  Max= "+sprintf(fmt,max(xCopy_avg_on_time)) )
print("-- xCopy_avg_on_time_stdev              (value) = "+sprintf(fmt,xCopy_avg_on_time_stdev))
print("  ")
print("   xCopy_ano(nT,:,:) = ( xCopy(nT,:,:) - xCopy_avg_on_time ) / xCopy_avg_on_time_stdev")
xCopy_ano=new((/dimsizes(xCopy)/),typeof(xCopy), default_fillvalue("integer"))
copy_VarMeta(xCopy, xCopy_ano)
do nT=0,dimsizes(xCopy&$timename$)-1
   xCopy_ano(nT,:,:) = ( xCopy(nT,:,:) - xCopy_avg_on_time ) / xCopy_avg_on_time_stdev
end do
print("-- xCopy_ano              (km2) rank= "+dimsizes(dimsizes(xCopy_ano)) \
		+"  Min= "+sprintf(fmt,min(xCopy_ano)) \
		+"  Max= "+sprintf(fmt,max(xCopy_ano)) )
;
print("  ")
;
print("2) Anomalies from total area --> (time)  ")
print("  ")
xCopy_total_area_avg   = avg(xCopy_total_area(ind_clm))
xCopy_total_area_stdev_on_time = dim_stddev(xCopy_total_area(ind_clm))
xCopy_total_area_stdev_on_fulltime = dim_stddev(xCopy_total_area)
print("-- xCopy_total_area_avg (km2) rank= "+dimsizes(dimsizes(xCopy_total_area_avg)) \
		+"  Min= "+sprintf(fmt,min(xCopy_total_area_avg)) \
		+"  Max= "+sprintf(fmt,max(xCopy_total_area_avg)) )
print("-- xCopy_total_area_stdev_on_time  (km2) = "+sprintf(fmt,(xCopy_total_area_stdev_on_time)) )
print("  ")
print("   xCopy_total_area_ano(nT) = ( xCopy_total_area(nT) - xCopy_total_area_avg )/xCopy_total_area_stdev_on_time")

xCopy_total_area_ano=new((/dimsizes(yyyy_sel)/),typeof(ww), default_fillvalue("integer"))
copy_VarMeta(data_3D_sea_inside_extract&$timename$, xCopy_total_area_ano)
xCopy_total_area_ano2020=new((/dimsizes(yyyy_sel)/),typeof(ww), default_fillvalue("integer"))
copy_VarMeta(data_3D_sea_inside_extract&$timename$, xCopy_total_area_ano2020)
do nT=0,dimsizes(yyyy_sel)-1
	xCopy_total_area_ano(nT) = ( xCopy_total_area(nT) - xCopy_total_area_avg )/xCopy_total_area_stdev_on_time
	xCopy_total_area_ano2020(nT) = (xCopy_total_area(nT) -  xCopy_total_area(ind_2018) ) /xCopy_total_area_stdev_on_fulltime
end do
xCopy_total_area_ano&$timename$ = xCopy_ano&$timename$ 
xCopy_total_area_ano2020&$timename$ = xCopy_ano&$timename$ 
print("-- xCopy_total_area_ano   (km2) rank= "+dimsizes(dimsizes(xCopy_total_area_ano)) \
		+"  Min= "+sprintf(fmt,min(xCopy_total_area_ano)) \
		+"  Max= "+sprintf(fmt,max(xCopy_total_area_ano)) )
print("  ")
;
; https://www.ncl.ucar.edu/Document/Functions/Built-in/runave.shtml
;*Calculates an unweighted running average on the rightmost dimension"
;  
; Use "runave" to compute [5-month] means (nave=5)
;
;  N = N-months - 1
;
;  opt < 0, cyclic conditions, my choice
;	x(0) = (xi(N) + xi(0) + xi(1) + xi(2)+ xi(3)) / nave 
;	x(N) = (xi(N-1) + xi(N) + xi(0) + xi(1)+  xi(2)) / nave
;
;  opt > 0, symmetric conditions, 
; 	x(0) = (xi(2) + xi(1) + xi(0) + xi(1)) / nave
; 	x(N) = (xi(N-2) + xi(N-1) + xi(N) + xi(N-1) + xi(N-2)) / nave")
;
print(getVarDimNames(xCopy_total_area_ano))
xCopy_total_area_ano_runave = runave_Wrap(xCopy_total_area_ano, 5, -1)      ; 5-mo run ave
;xCopy_total_area_ano_runave&$timename$ = xCopy_ano&$timename$ 
print("-- xCopy_total_area_ano_runave   (km2) rank= "+dimsizes(dimsizes(xCopy_total_area_ano_runave)) \
		+"  Min= "+sprintf(fmt,min(xCopy_total_area_ano_runave)) \
		+"  Max= "+sprintf(fmt,max(xCopy_total_area_ano_runave)) )
return([/xCopy_ano,xCopy_total_area_ano,xCopy_total_area_ano2020, xCopy_total_area_ano_runave /])
end
;
;*************************************************
;*************************************************
undef("fct_res_tserie")
function fct_res_tserie(YAxisString,XAxisString)
begin
rts             = True
rts@gsnMaximize = True                 ; make ps/eps/pdf large
rts@gsnDraw     = False       ; don't draw yet
rts@gsnFrame    = False       ; don't advance frame yet
rts@gsnScale    = False      ; force text scaling               
rts@gsnYRefLine = 0.         ; reference line   

rts@xyDashPattern     = 0                         ; Make curves all solid

rts@trYMinF            =  -4.                          ;-- y-axis minimum
rts@trYMaxF            =  4.                          ;-- y-axis minimum

rts@tiMainFontHeightF   = 0.025

rts@tiYAxisString = YAxisString
rts@tiYAxisFontHeightF = 0.03
rts@tiXAxisString = XAxisString
rts@tiXAxisFontHeightF = 0.03
return(rts)
end
;
;*************************************************
;*************************************************
undef("plt_Fig3")
procedure plt_Fig3(pltName,expName,region,season,var,yyyy_sel,ind_ano,ind_ano2020)
local expName, region
begin
; https://www.ncl.ucar.edu/Applications/xy.shtml
wks = gsn_open_wks("png",pltName)

YAxisString = season+"  "+var+" Anomaly "
XAxisString = " Year "
rts = fct_res_tserie(YAxisString,XAxisString)
rts@tiMainString= expName+"-ERA5"
rts@vpHeightF   = 0.40        ; Changes the aspect ratio
rts@vpWidthF    = 0.75
rts@vpXF        = 0.10        ; change start locations
rts@vpYF        = 0.75        ; the plot

rts@xyLineColors      = "black"
rts@xyLineThicknessF  = 3.

plot = gsn_csm_xy (wks,yyyy_sel,ind_ano,rts)

circle   = new(dimsizes(yyyy_sel),graphic)
polyres                    = True                       ; marker resources
polyres@gsMarkerIndex      = 16                        	; 16, for plein circle
polyres@gsMarkerSizeF      = .01                        ; size 
polyres@gsMarkerThicknessF = 1.5                 	; Twice as thick
polyres@gsMarkerColor      = "black"
do nT=0,dimsizes(yyyy_sel)-1
   circle(nT)   = gsn_add_polymarker(wks,plot,yyyy_sel(nT),ind_ano2020(nT),polyres)
end do
draw(plot)
frame(wks)
end
;
;*************************************************
;*************************************************
undef("plt_Fig4")
procedure plt_Fig4(pltName,expName,region,season,var,yyyy_sel,ind_ano,ind_ano2020)
local expName, ind_ano, ind_ano2020
; https://www.ncl.ucar.edu/Applications/xy.shtml
begin
wks = gsn_open_wks("png",pltName)

YAxisString = season+"  "+var+" Anomaly "
XAxisString = " Year "
rts = fct_res_tserie(YAxisString,XAxisString)
rts@tiMainString= expName
rts@gsnYRefLineDashPattern  =  1

rts@xyLineColors      = (/"black","red"/)          ; change line color
rts@xyLineThicknessF  = (/ 8.0 , 8.0 /)
;
; plot of lines
plot = gsn_csm_xy (wks,yyyy_sel,ind_ano,rts)
;
; plot of circles
circle   = new((/2,dimsizes(yyyy_sel)/),graphic)
;
polyres                   = True                       ; marker resources
polyres@gsMarkerIndex     = 4                          ; 16, for plein circle
polyres@gsMarkerSizeF     = .014                        ; size 
polyres@gsMarkerThicknessF = 2.
;
polyres@gsMarkerColor      = "black"
do nT=0,dimsizes(yyyy_sel)-1
   circle(0,nT)   = gsn_add_polymarker(wks,plot,yyyy_sel(nT),ind_ano2020(0,nT),polyres)
end do
;
polyres@gsMarkerColor      = "red"
do nT=0,dimsizes(yyyy_sel)-1
   circle(1,nT)   = gsn_add_polymarker(wks,plot,yyyy_sel(nT),ind_ano2020(1,nT),polyres)
end do
;
;add marker size legend
txres = True
txres@gsFontColor = "black"
txres@txFontHeightF = 0.02
;
markx = .25
marky = .24
;
polyres@gsMarkerIndex = 16 			;-- filled dots
;
gsn_polymarker_ndc(wks,markx,marky,polyres)
gsn_text_ndc(wks,"Eurasian Arctic",markx+.12,marky,txres)
;
polyres@gsMarkerColor      = "black"
gsn_polymarker_ndc(wks,markx,marky+.04,polyres)
gsn_text_ndc(wks,"North American Arctic",markx+.16,marky+.04,txres)
;

draw(plot)
frame(wks)
;
end
;*************************************************
; Main Program
;*************************************************
begin
setfileoption("nc","MissingToFillValue",False)
quote = str_get_dq()
cr = inttochar(10)

pltType= "png"                                                 ; x11, ps, pdf, png
pltCol = "amwg256"

var = "snct"
varString = "Snow Cover Extent "
print(cr+"_________________________________________ "+ varString)

if .not.isvar("USE_LOCAL") then USE_LOCAL = True  end if

dirou = "./tmp/"
expeString = (/"CROCUS-ERA5","CROCUS-ERAI"/) 
;period     = (/ "1950-2020", "1979-2019" /)
period     = (/ "1967-2020", "1979-2019" /)
regions     = (/ "NAmerica", "Eurasia" /)
;
; selection of period overlapping both experiements and multiple of 12
yrStrt="1979"
yrLast="2018"

if .not.isvar("nexp") then nexp=0 end if
;
if expeString(nexp).eq."CROCUS-ERA5" then 
   fname=var+"_HOMALmon_CROCUS-ISBA_ideal-era5_r1i1p1f2_gn_195001-202306.nc" 
end if
if expeString(nexp).eq."CROCUS-ERAI" then 
   fname="crocusv8.0_ideal_erai_arpsfx_monthly_"+var+"_"+period(nexp)+".nc"
end if
;
if USE_LOCAL then
    root = "./"
    if expeString(nexp).eq."CROCUS-ERA5" then
        path_exp="./"+fname
    end if
    if expeString(nexp).eq."CROCUS-ERAI" then 
        path_exp="CROCUSv8.0/ideal-erai/output/"+fname 
    end if
else
    root = "./"
    if expeString(nexp).eq."CROCUS-ERA5" then
        path_exp="CROCUS-ERA5/ideal-era5/r1i1p1f2/output/"+fname
    end if
    if expeString(nexp).eq."CROCUS-ERAI" then 
        path_exp="CROCUSv8.0/ideal-erai/output/"+fname 
    end if
end if

if .not.isvar("nvar") then nvar=1 end if

;
mthstr = (/"JAN","FEB","MAR","APR","MAY","JUN","JUL","AUG","SEP","OCT","NOV","DEC"/)
seastr = (/"DJF","JFM","FMA","MAM","AMJ","MJJ","JJA","JAS","ASO","SON","OND","NDJ"/)
;
if .not.isvar("season") then season="ALL" end if
;
pltName   = dirou+expeString(nexp)+"_"+var+"_" + season + "_" + "_masked"
dYYYY = (stringtoint(yrLast) - stringtoint(yrStrt))
MainTitle= expeString(nexp)+" "+season +" ("+dYYYY +")" 
if (season.eq."ALL") then
   MainTitle= expeString(nexp)+" ("+dYYYY +")" 
end if
;
;--- Source File
;
print(" --> "+root+path_exp)
fY  = addfile(root+path_exp,"r")
xc = fY->$var$
att = getVarDimNames(xc)
timename  = att(0)
latname   = att(1)
lonname   = att(2)
;
;--- Calculation of time intervals
;
result=goodtime(xc&$timename$,yrStrt,yrLast)          ; includes meta data
YYYY   = result[0]                              ; MAJ point to total period
iyyyy  = result[1]
YYYYMM = result[2]
delete(result)
;
PeriodString = "("+YYYY(0)+"-"+YYYY(dimsizes(YYYY)-1)+")"
print(". we have: "+PeriodString+" N° months: "+dimsizes(YYYY))
yyyymm = YYYYMM(iyyyy)
PeriodString = "("+YYYY(iyyyy(0))+"-"+YYYY(iyyyy(dimsizes(iyyyy)-1))+")"
print(". we want: "+PeriodString+" N° months: "+dimsizes(YYYY(iyyyy)))
; 
print("  ")
print("-- YYYY            min= "+min(YYYY)+" max= "+max(YYYY)+" N° months: "+dimsizes(YYYYMM))
print("-- YYYY(iyyyy)     min= "+min(YYYY(iyyyy))+" max= "+max(YYYY(iyyyy))+" N° months: "+dimsizes(YYYY(iyyyy)) )
; 
; time axis of selected full time-period
yyyy_sel = ispan(min(YYYY(iyyyy)),max(YYYY(iyyyy)),1)
;
; climatological basis relative to the 1981-2010 period
result=goodtime(xc&$timename$,"1981","2000")          ; includes meta data
iyyyy_sel  = result[1]
PeriodString = "("+YYYY(iyyyy_sel(0))+"-"+YYYY(iyyyy_sel(dimsizes(iyyyy_sel)-1))+")"
print("-- YYYY(iyyyy_sel) min= "+min(YYYY(iyyyy_sel))+" max= "+max(YYYY(iyyyy_sel))+" N° months: "+dimsizes(YYYY(iyyyy_sel)) )
delete(result)
print("  ")
;
;--- Selection of monthly/seasonly/annually time-serie, here time axis change
;
data_3D = xc(iyyyy,:,:)				; full time-serie requested
copy_VarAtts(xc,data_3D)
if .not.ismissing(str_match(var,"snc")) then
	data_3D = data_3D * 100.				; full time-serie requested
end if
print("-- xc           min= "+min(xc)+" max= "+max(xc)+" N° months: "+dimsizes(xc&$timename$))
print("-- data_3D      min= "+min(data_3D)+" max= "+max(data_3D)+" N° months: "+dimsizes(data_3D&$timename$))
delete(xc)
;
; monthly extraction
if .not.ismissing(str_match(mthstr,season)) then
;  	ii + 1 --> because of place list's names are counted from 0
;  	whereas,
;  	mm     --> order number of months, counted from 1
   ii = str_match_ind(mthstr,season) + 1	; records's right places - counted initially from 0
   cyyyy = floor(yyyymm/100)
   mm = toint(yyyymm - (cyyyy*100))		
   imm = ind(mm.eq.ii)  		
   data_3D_sea = data_3D(imm,:,:)
   data_3D_sea&$timename$ = yyyymm(imm)/100
   ;print("  "+yyyymm(imm))          		; yyyymm records's right places are counted from 1
end if
;
; seasonly extraction
if .not.ismissing(str_match(seastr,season)) then
   data_3D_sea = month_to_season(data_3D, season)
   data_3D_sea&$timename$ = yyyy_sel
end if
; seasonly extraction
if (season.eq."ALL") then
; ERROR message --> https://www.ncl.ucar.edu/Support/talk_archives/2013/2073.html
; It happens because some grid points have missing values, which is not
; surprising since there shouldn't be values over ocean.
  data_3D_sea = month_to_annual(data_3D,1)
  data_3D_sea!0 = timename
  data_3D_sea&$timename$ = yyyy_sel
end if
if(.not.isatt(data_3D_sea,"_FillValue")) then  data_3D_sea@_FillValue = 1e+20  end if
printMinMax(data_3D_sea,0)
;
;--- Masking data 
;
region       = (/"NAmerica","Eurasia","Greenland"/) 
;
; NORTH AMERICA
;
print(cr+"*** "+region(0)+" ***")
masks_data  = (/"Canada","UnitedStates"/)
masks_data_shortName = (/"CAN","USA"/)
;
nmax=dimsizes(masks_data)
do nm=0, nmax-1
    print(var+"_"+masks_data(nm))
    fm    = addfile("~/tong/dat/"+masks_data(nm)+"_mask_"+expeString(nexp)+".nc","r")
    mask_country = fm->mask_array
    country_inside = mask(data_3D, mask_country.eq.0,False)
    country_sea_inside = mask(data_3D_sea, mask_country.eq.0,False)
    if nm.eq.0 then 
        data_3D_sea_inside = country_sea_inside 
    else
        do t=0, dimsizes(data_3D_sea&$timename$)-1
            data_3D_sea_inside(t,:,:) = where(mask_country.eq.1.,data_3D_sea(t,:,:),data_3D_sea_inside(t,:,:))
        end do 
    end if
end do
copy_VarMeta(data_3D_sea,data_3D_sea_inside)
;
data_3D_sea_inside_extract = data_3D_sea_inside(:,{60:},:)  		; region extract 
lat1d    = data_3D_sea_inside_extract&$latname$
lon1d    = data_3D_sea_inside_extract&$lonname$
lat1d@_FillValue = default_fillvalue("integer")
lon1d@_FillValue = default_fillvalue("integer")
;
RESULT = get_wgt(lat1d,lon1d)					; calculation of areas
area   = RESULT[0]
delete(RESULT)
print("  ")
;
; Fig1
data_3D_sea_ave = dim_avg_n_Wrap(data_3D_sea,0) 
Clm_period=min(data_3D_sea&$timename$)+"-"+max(data_3D_sea&$timename$)
pltName=dirou+"Fig1_"+var+"_"+expeString(nexp)+"_"+region(0)+"_masked_"+season+"_"+Clm_period
data_3D_sea_inside_extract_ave = dim_avg_n_Wrap(data_3D_sea_inside_extract,0) 
goodLevels = fct_get_Levels("posLab5")
plt_Fig1(pltName,lat1d,lon1d,data_3D_sea_ave,data_3D_sea_inside_extract_ave,goodLevels)
print("-- data_3D_sea_inside (km2) rank= "+dimsizes(dimsizes(data_3D_sea_inside))+", min= "+min(data_3D_sea_inside)+" max= "+max(data_3D_sea_inside))

;
result = get_ind(area,yyyy_sel,data_3D_sea_inside_extract,expeString(nexp),region(0),cr)
xCopy_ano    = result[0]
xCopy_total_area_ano = result[1]
;
; To plot multiple lines, you must put them into a mulidimensional array.
xCopy_total_area_ano2020_both = new((/2,dimsizes(result[2])/),typeof(result[2]))
xCopy_total_area_ano2020_both(0,:) = result[2]
xCopy_total_area_ano_runave_both = new((/2,dimsizes(result[3])/),typeof(result[3]))
xCopy_total_area_ano_runave_both(0,:) = result[3]
;
; Fig3
pltName=dirou+"Fig3_"+expeString(nexp)+"-"+region(0)+"_"+season+"_tserie"
plt_Fig3(pltName,expeString(nexp),region(0),season,var,yyyy_sel,xCopy_total_area_ano_runave_both(0,:),xCopy_total_area_ano2020_both(0,:))
;
; Stats on elementary surface (!)
fStat = expeString(nexp)+"_"+var+"_"+season+"_"+yrStrt +"-"+ yrLast+"_"+region(0)+".dat"
asciiPath = "~/tong/ana/stats_"+fStat
print("Stats  ["+var+"] --->  "+ asciiPath)
print(getVarDimNames(xCopy_ano))
;get_statFile(asciiPath,xCopy_ano)
get_statFile(asciiPath,data_3D_sea_inside_extract)
;
delete(masks_data)
delete(masks_data_shortName)
delete(data_3D_sea_inside)
;
;
; EURASIA
; https://gadm.org/download_country.html
;
print(cr+"*** "+region(1)+" ***")
masks_data       = (/"Russia","Finland","Iceland","Norway","Sweden"/)
masks_data_shortName = (/"RUS","FIN","ISL","NOR","SWE"/)
nmax=dimsizes(masks_data)
do nm=0, nmax-1
    print(var+"_"+masks_data(nm))
    fm    = addfile("~/tong/dat/"+masks_data(nm)+"_mask_"+expeString(nexp)+".nc","r")
    mask_country = fm->mask_array
    country_inside = mask(data_3D, mask_country.eq.0,False)
    country_sea_inside = mask(data_3D_sea, mask_country.eq.0,False)
    if nm.eq.0 then 
        data_3D_sea_inside = country_sea_inside 
    else
        do t=0, dimsizes(data_3D_sea&$timename$)-1
            data_3D_sea_inside(t,:,:) = where(mask_country.eq.1.,data_3D_sea(t,:,:),data_3D_sea_inside(t,:,:))
        end do 
    end if
end do
copy_VarMeta(data_3D_sea,data_3D_sea_inside)
;
data_3D_sea_inside_extract = data_3D_sea_inside(:,{60:},:)  		; region extract 
;
; Fig1
data_3D_sea_ave = dim_avg_n_Wrap(data_3D_sea,0) 
Clm_period=min(data_3D_sea&$timename$)+"-"+max(data_3D_sea&$timename$)
pltName=dirou+"Fig1_"+var+"_"+expeString(nexp)+"_"+region(1)+"_masked_"+season+"_"+Clm_period
data_3D_sea_inside_extract_ave = dim_avg_n_Wrap(data_3D_sea_inside_extract,0) 
goodLevels = fct_get_Levels("posLab5")
plt_Fig1(pltName,lat1d,lon1d,data_3D_sea_ave,data_3D_sea_inside_extract_ave,goodLevels)
print("-- data_3D_sea_inside (km2) rank= "+dimsizes(dimsizes(data_3D_sea_inside))+", min= "+min(data_3D_sea_inside)+" max= "+max(data_3D_sea_inside))
;
result = get_ind(area,yyyy_sel,data_3D_sea_inside_extract,expeString(nexp),region(1),cr)
xCopy_ano    = result[0]
xCopy_total_area_ano = result[1]
xCopy_total_area_ano2020_both(1,:) = result[2]
xCopy_total_area_ano_runave_both(1,:) = result[3]
;
; plt_Fig3
pltName=dirou+"Fig3_"+var+"_"+expeString(nexp)+"-"+region(1)+"_"+season+"_tserie"
plt_Fig3(pltName,expeString(nexp),region(1),season,var,yyyy_sel,xCopy_total_area_ano_runave_both(1,:),xCopy_total_area_ano2020_both(1,:))
;
; plt_Fig4
;
pltName=dirou+"Fig4_"+var+"_"+expeString(nexp)+"_"+season+"_tserie"
plt_Fig4(pltName,expeString(nexp),region,season,var,yyyy_sel,xCopy_total_area_ano_runave_both,xCopy_total_area_ano2020_both)
;
; Stats on elementary surface (!)
fStat = expeString(nexp)+"_"+var+"_"+season+"_"+yrStrt +"-"+ yrLast+"_"+region(1)+".dat"
asciiPath = "~/tong/ana/stats_"+fStat
print("Stats  ["+var+"] --->  "+ asciiPath)
print(getVarDimNames(xCopy_ano))
get_statFile(asciiPath,xCopy_ano)
;
delete(masks_data)
delete(masks_data_shortName)
delete(data_3D_sea_inside)

;
; GREENLAND
;
print(cr+"*** "+region(2)+" ***")
regFName = systemfunc("echo "+region(2)+"| tr 'G' 'g' ")
fm    = addfile("~/tong/dat/"+regFName+"_mask_"+expeString(nexp)+".nc","r")
mask_country = fm->mask_array
;
data_inside = mask(data_3D,mask_country.eq.0,False)		; masked data
copy_VarMeta(data_3D,data_inside)
;
data_3D_sea_inside = mask(data_3D_sea,mask_country.eq.0,False)	; masked seasonal extract
copy_VarMeta(data_3D_sea,data_3D_sea_inside)
if(.not.isatt(data_3D_sea_inside,"_MissingValue")) then data_3D_sea_inside@_MissingValue = data_3D@_FillValue end if
;
data_3D_sea_inside_extract = data_3D_sea_inside(:,{60:},:)  		; region extract 
lat1d    = data_3D_sea_inside_extract&$latname$
lon1d    = data_3D_sea_inside_extract&$lonname$
lat1d@_FillValue = default_fillvalue("integer")
lon1d@_FillValue = default_fillvalue("integer")
;
RESULT = get_wgt(lat1d,lon1d)					; calculation of areas
area   = RESULT[0]
delete(RESULT)
print("  ")
;
; Fig1
data_3D_sea_ave = dim_avg_n_Wrap(data_3D_sea,0) 
Clm_period=min(data_3D_sea&$timename$)+"-"+max(data_3D_sea&$timename$)
pltName=dirou+"Fig1_"+var+"_"+expeString(nexp)+"_"+region(2)+"_masked_"+season+"_"+Clm_period
goodLevels = fct_get_Levels("posLab5")
data_3D_sea_inside_extract_ave = dim_avg_n_Wrap(data_3D_sea_inside_extract,0) 
plt_Fig1(pltName,lat1d,lon1d,data_3D_sea_ave,data_3D_sea_inside_extract_ave,goodLevels)
print("-- data_3D_sea_inside (km2) rank= "+dimsizes(dimsizes(data_3D_sea_inside))+", min= "+min(data_3D_sea_inside)+" max= "+max(data_3D_sea_inside))
;
result = get_ind(area,yyyy_sel,data_3D_sea_inside_extract,expeString(nexp),region(2),cr)
xCopy_ano    = result[0]
xCopy_total_area_ano = result[1]
xCopy_total_area_ano2020 = result[2]
xCopy_total_area_ano_runave = result[3]
;
; plt_Fig3
pltName=dirou+"Fig3_"+expeString(nexp)+"-"+region(2)+"_"+season+"_tserie"
plt_Fig3(pltName,expeString(nexp),region(2),season,var,yyyy_sel,xCopy_total_area_ano_runave,xCopy_total_area_ano2020)
;
; Stats on elementary surface (!)
fStat = expeString(nexp)+"_"+var+"_"+season+"_"+yrStrt +"-"+ yrLast+"_"+region(2)+".dat"
asciiPath = "~/tong/ana/stats_"+fStat
print("Stats  ["+var+"] --->  "+ asciiPath)
print(getVarDimNames(xCopy_ano))
get_statFile(asciiPath,xCopy_ano)
;
end
