#!/usr/bin/env python
# -*- coding:UTF-8 -*-
#
# Created On: 2023
# Created By: S. Buarque

import os
import sys
from netCDF4 import Dataset as netcdf_ds

# Minimum dependency versions to xarray
import xarray as xr
import pandas as pd
import numpy as np

import matplotlib.pyplot as plt
import matplotlib.colors as mcolors

import glob

import seaborn as sns

import calendar
import locale

import cartopy.crs as ccrs
import cartopy.feature as cfeature

#-------------------------------
# Default Parameters
#-------------------------------
locale.setlocale(locale.LC_TIME, 'en_DK.UTF-8')
print("+ + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + ")
print([calendar.month_abbr[i].lower() for i in range(13)])

USE_LOCAL = True

dir_out='./tmp/'
tperiod = ['2000-01-01', '2022-12-31']

dataset = ['CROCUS-ERA5', 'IMS']
nmonth=0

cmap='Blues_r'

#-------------------------------
# Functions
#-------------------------------
def get_path_in(USE_LOCAL, expe ):
    if USE_LOCAL : 
       root = './'
    else:
       root = "./remote_dir/"
    varname = 'snct' 
    if expe ==  'IMS' : 
        varname   = "IMS_Surface_Values"
        expe_path = ""
        fname     = "ims2000-2023_monthly_means_00UTC_24km_v1.3.nc"
        path_in=os.path.join(root,expe_path,fname)
    if expe ==  'CROCUS-ERA5' : 
        expe_path=""
        var = varname+'_HOMALmon' 
        path_in=os.path.join(root,expe_path,var+'_CROCUS-ISBA_ideal-era5_r1i1p1f2_gn_195001-202306.nc')
    return varname, path_in



def center_longitudes(lon, range_180=True):
    """
    Center longitudes to the range [-180, 180] or [0, 360].

    Parameters:
    lon (array-like): Array of longitude values.
    range_180 (bool): If True, convert to range [-180, 180]. If False, convert to range [0, 360].

    Returns:
    np.ndarray: Array of centered longitude values.
    """
    lon = np.array(lon)
    if range_180:
        lon = ((lon + 180) % 360) - 180
    else:
        lon = lon % 360
    return lon


def correct_longitudes(lon):
    """
    Correct longitudes by adding 180 degrees and normalizing to [-180, 180].

    Parameters:
    lon (array-like): Array of longitude values.

    Returns:
    np.ndarray: Array of corrected longitude values.
    """
    # Add 180 degrees to shift longitudes
    lon_corrected = lon + 180

    # Normalize to [-180, 180] range
    lon_corrected = ((lon_corrected + 180) % 360) - 180

    return lon_corrected


def plot_map(cmap,varplot,plotname,gtime):
    fig, ax = plt.subplots(figsize=(12, 4))
    varplot.plot(ax=ax, cmap=cmap)
    ax.set_title(f'Monthly Fractional Snow Cover: {gtime}')
    ax.set_xlabel('Longitude')
    ax.set_ylabel('Latitude')
    ax.set_facecolor('gray')
    plt.savefig('./tmp/'+plotname+"_"+gtime)
    plt.clf()
        

def wgt_change3Dt1D(data3D, latname, lonname):
    print("\n======================== wgt_change3Dt1D")
    latitude = data3D[latname]
    taille_cellule = np.cos(np.deg2rad(latitude))

    # Compute the mean and standard deviation of each dataset
    mu_1D_wgt = data3D.weighted(taille_cellule).mean([latname,lonname])
    print("data3D     coords : ", data3D.coords, "\n")
    print("mu_1D_wgt coords : ", mu_1D_wgt.coords, "\n")
    print("mu_1D_wgt        : ", mu_1D_wgt.shape)
    print("mu_1D_wgt class  : ", mu_1D_wgt.__class__)

    std1  = data3D.weighted(taille_cellule).std([latname,lonname])
    print("std1 shape : ",std1.shape)

    var_1D_wgt = mu_1D_wgt/std1

    print("======================== wgt_change3Dt1D END\n")
    return var_1D_wgt


def anomaly1D(da_fsc, latname, lonname):
    print("\n======================== anomaly1D")
    # Calculate the climatology (monthly mean)
    da_clim3D = da_fsc.groupby('time.month').mean('time')

    # Calculate the anomalies
    da_anom3D = da_fsc.groupby('time.month') - da_clim3D

    # Plot the climatology
    gtime = str(da_anom3D['time'].values[nmonth]).split('T')[0]
    plot_map(cmap, da_clim3D[nmonth], dataset[i], gtime)

    # Compute the 1D anomaly
    da_anom1D = da_anom3D.mean([latname, lonname])
    print("da_anom1D class: ", da_anom1D.__class__)

    # Weighting data with weighted latitude
    da_anom1D_wgt = wgt_change3Dt1D(da_anom3D, latname, lonname)

    # Ensure the DataArray has a name
    da_anom1D.name = 'anomaly'
    da_anom1D_wgt.name = 'anomaly'

    print("\nda_anom1D_wgt shape  : ", da_anom1D_wgt.shape)
    print("da_anom1D_wgt coords : ",da_anom1D_wgt.coords)

    print("da_anom1D_wgt class  : ", da_anom1D_wgt.__class__)

    print("\n======================== anomaly1D END")
    return  da_anom1D_wgt


def interpolation(tmp):
    # Extract longitudes and identify duplicates
    lon = tmp['lon'].values
    unique_lon, indices, counts = np.unique(lon, return_index=True, return_counts=True)

    # Initialize a new DataArray to store the aggregated data with unique longitudes
    new_data = np.zeros((tmp.shape[0], tmp.shape[1], len(unique_lon)))  # Adjust shape if needed

    # Loop through unique longitudes and handle duplicates
    for i, ulon in enumerate(unique_lon):
        matching_indices = np.where(lon == ulon)[0]
        if len(matching_indices) > 1:
            # Average data across duplicate longitudes
            aggregated_data = tmp.isel(lon=matching_indices).mean(dim='lon')
        else:
            aggregated_data = tmp.isel(lon=matching_indices[0])
        new_data[:, :, i] = aggregated_data

    # Create a new DataArray with unique longitudes
    tmp_unique = xr.DataArray(
        new_data,
        coords={
            'time': tmp['time'],
            'lat': tmp['lat'],
            'lon': unique_lon
        },
        dims=['time', 'lat', 'lon']
    )

    # Check uniqueness
    assert len(np.unique(tmp_unique['lon'])) == len(tmp_unique['lon']), "There are still duplicate longitudes"

    # Perform interpolation using correct dimension names
    tmp = tmp_unique.interp(lat=lat_ref, lon=lon_ref, method='linear')

    return tmp


def get_fcsv(csvfilename, var1D ):
    print("\n======================== get_fcsv")

#-- Check coordinates of the intial DataArra
    print("var1D: ",var1D.coords)

#-- transform DataArray to a xarray.DataFrame which, in principle, can be written to a CSV file.
    df_var1D = var1D.to_dataframe()
    print("df_var1D  : ", df_var1D.__class__)

#-- months as english abreviations")
    df_var1D['month'] = df_var1D['month'].apply(lambda x: calendar.month_abbr[x].lower())

    # Save the DataFrame to a CSV file
    #df_var1D.to_csv(csvfilename, date_format='%Y-%M-%d', float_format='%.3f', sep =' ')
    df_var1D.to_csv(csvfilename, date_format='%Y', float_format='%.3f', sep =' ')
    print("======================== get_fcsv END\n")
    return

    

#-------------------------------
# Main
#-------------------------------
varname, path = get_path_in(USE_LOCAL, 'CROCUS-ERA5' )
da_ref = xr.open_dataset(path)[varname]
lat_ref = da_ref['lat'][:]
lon_ref = da_ref['lon'][:]

lat_min=25
#
fcsv=[]
for i in range(len(dataset)) :
    print("\n=========================================================================== \n"+dataset[i])
    varname, path = get_path_in(USE_LOCAL, dataset[i] )
    da_all = xr.open_dataset(path)[varname]
    da_fsc = da_all.sel(time=slice(tperiod[0],tperiod[1]))

    # Print dataset attributes to check for missing value indicators

    # Try to get the _FillValue or missing_value attribute, default to NaN if not found
    fill_value = da_fsc.attrs.get('_FillValue', np.nan)

    # Check for other potential missing value indicators (e.g., 'missing_value')
    missing_value = da_fsc.attrs.get('missing_value', np.nan)

    # Replace missing values with NaN
    da_fsc = da_fsc.where(da_fsc != missing_value, np.nan)
    print("\nda_fsc coords: ",da_fsc.coords)
    print("da_fsc shape : ",da_fsc.shape, "\n")

    if dataset[i] == 'CROCUS-ERA5' :
        fcsv.append("./"+dataset[i]+"_"+varname+".csv")
        da_crc_fsc = da_fsc
        da_lat=da_fsc.lat
        da_lon=da_fsc.lon

        ce5_anom1D_wgt = anomaly1D(da_fsc, 'lat', 'lon')

        get_fcsv(fcsv[i], ce5_anom1D_wgt)


    if dataset[i] == 'IMS' :
        fcsv.append("./"+varname+".csv")
        # Correct longitudes then Ensure longitudes and latitudes are sorted
        da_fsc['lon'] = center_longitudes(da_fsc['lon'], range_180=True)
        da_fsc['lon'] = correct_longitudes(da_fsc['lon'].values)
        da_fsc = da_fsc.sortby('lon').sortby('lat')

        # Filter latitudes and corresponding snow cover data above lat_min (default = 25°N)
        lat_mask = da_fsc.lat >= lat_min
        filtered_latitudes = da_fsc.lat[lat_mask]

        # Adjust the fractional_snow_cover matrix to match filtered latitudes
        da_fsc = da_fsc[:,lat_mask, :]

        da_lat=da_fsc.lat
        da_lon=da_fsc.lon

        # Rename coordinates to match expected names
        #da_fsc = da_fsc.rename({ 'latitude': 'lat', 'longitude': 'lon'})
        print("\nda_fsc coords: ",da_fsc.coords)

        # Check for uniqueness
        assert len(np.unique(da_lat)) == len(da_lat), "da_lat contains duplicate values"
        da_ims_fsc = interpolation(da_fsc)
        print("\nInterpolated DataArray:", da_ims_fsc.coords)

        ims_anom1D_wgt = anomaly1D(da_ims_fsc, 'lat', 'lon')

        get_fcsv(fcsv[i], ims_anom1D_wgt)

#
print("\n",i,  path,"\n",  fcsv[i], "\n")

print("======================== Time-serie\n")

# Create the figure and axis
fig, ax = plt.subplots(1, 1, figsize=(14, 4))

# Plot the datasets with their label
ce5_anom1D_wgt.plot(label=' CROCUS-ERA5 ', color='black',  linewidth=1 , alpha=0.8)
ims_anom1D_wgt.plot(label=' IMS ', color='red',  linewidth= 0.8, alpha=0.7)

ax.legend()
plt.savefig(dir_out + 'CROCUS-' + varname + '_fromLat' + str(lat_min) + '_anomaly_v0.png')

# Linear regression to draw a trend line fr
# Assuming ce5_anom1D_wgt and ims_anom1D_wgt are xarray.DataArray objects with a time coordinate
#
from scipy.stats import linregress

# Extract time coordinates as numeric values (e.g., days since start)
time_numeric = (ce5_anom1D_wgt.coords['time'] - ce5_anom1D_wgt.coords['time'][0]).astype('timedelta64[D]').astype(int)

# Perform linear regression
ce5_slope, ce5_intercept, ce5_rvalue, _, ce5_stderr = linregress(time_numeric, ce5_anom1D_wgt)
ims_slope, ims_intercept, ims_rvalue, _, ims_stderr = linregress(time_numeric, ims_anom1D_wgt)

# Calculate the regression lines
ce5_trend = ce5_slope * time_numeric + ce5_intercept
ims_trend = ims_slope * time_numeric + ims_intercept


# Plot the linear regression lines
ax.plot(ce5_anom1D_wgt.coords['time'], ce5_trend, color='black', linewidth=1,  linestyle='dashed'  )
ax.plot(ims_anom1D_wgt.coords['time'], ims_trend, color='red', linewidth=1,  linestyle='dashed' )
#ax.plot(ce5_anom1D_wgt.coords['time'], ce5_trend, color='black', label='CE5 Linear Fit', linewidth=1,  linestyle='dashed'  )
#ax.plot(ims_anom1D_wgt.coords['time'], ims_trend, color='red', label='IMS Linear Fit', linewidth=1,  linestyle='dashed' )

# Add a legend to the plot
ax.legend()

# Add labels and title
ax.set_xlabel('Time')  # Replace with your actual x-axis label if needed
ax.set_ylabel('Standardized Anomalies')  # Replace with your actual y-axis label
ax.set_title('Snow Cover Extent')

#a - - - - - - - - - - - -  -
# Calculate the correlation coefficient
# Convert xarray.DataArray to pandas.Series
ce5_series = ce5_anom1D_wgt.to_series()
ims_series = ims_anom1D_wgt.to_series()

# Align the series to ensure they have the same time index
ce5_series, ims_series = ce5_series.align(ims_series, join='inner')


# Save the plot
plt.savefig(dir_out + 'CROCUS-' + varname + '_fromLat' + str(lat_min)+ '_anomaly.png')
plt.clf()

print("\n",dir_out + 'CROCUS-' + varname + '_fromLat' + str(lat_min)+ '_anomaly.png')
exit()

